program maxprofit;

  { the program reads an input file input.txt and creates an output file output.txt }

const
  maxm = 34;
  maxstep = 10;
  maxks = 5260;
  tol1 = 1.0e-06;
  tol2 = 1.0e-07;
  tol3 = 1.0e-06;
  tol4 = 1.0e-05;
  minweight = 1.0;
  minreal = -1.0e+10;
  maxreal = 1.0e+10;

type
  ivector = array [1..maxm] of integer;

  rvector = array [1..maxm] of single;

  errortype = (unbounded, singular, outbound1, outbound2);

  statustype = (bas, low, upp);

  demandtype = record
    width1: integer;
    width2: integer;
    quantity: single;
    alpha: single;
    theta: single
  end;

var
  nocutoff: boolean;
  m, m1, m2, nd, width, widthks, step, iteration: integer;
  prodprice, prodcost, scrapprice, obj: single;
  equation, basecol, wks, o, change, w: ivector;
  upper: array [0..maxm] of single;
  v, xb, b, q, p, cb, r, a: rvector;
  aa: array [1..2, 1..maxm] of single;
  base, d: array [1..maxm, 1..maxm] of single;
  status: array [0..maxm] of statustype;
  demand: array [1..maxm] of demandtype;

procedure errormsg(error: errortype);
begin
  case error of
    unbounded: write('unbounded optimal');
    singular: write('pivot lower than tol1');
    outbound1: write('adjust maxm');
    outbound2: write('adjust maxks')
  end;
  writeln;
  
  write('execution aborted');
  writeln;
  halt
end;

procedure update(var leftstatus: statustype; var enter, leave: integer; var a: rvector);
var
  i, j, k, imax, iaux, row1, row2: integer;
  sum, aux: single;

begin
  { update status, basecol, base, cb }
  status[basecol[leave]] := leftstatus;
  status[enter] := bas;
  basecol[leave] := enter;
  for i := 1 to m do
    base[i, leave] := a[i];
  if enter > 0 then
    cb[leave] := 0.0
  else
  begin
    aux := 0.0;
    for i := 1 to m do
      aux := aux + r[i] * a[i];
    cb[leave] := width - aux
  end;

  { update change, equation, d }
  for k := m - 1 downto leave do
  begin
    row1 := k;
    row2 := change[k];
    if row2 > 0 then
    begin
      change[k] := 0;

      iaux := equation[row1];
      equation[row1] := equation[row2];
      equation[row2] := iaux;

      for j := 1 to leave - 1 do
      begin
        aux := d[row1, j];
        d[row1, j] := d[row2, j];
        d[row2, j] := aux
      end
    end
  end;

  { update d }
  for i := 1 to m do
    d[i, leave] := a[equation[i]];
  for j := leave + 1 to m do
    for i := leave to m do
      d[i, j] := base[equation[i], j];

  { column leave }
  for i := 1 to leave do
  begin
    sum := d[i, leave];
    for k := 1 to i - 1 do
      sum := sum - d[i, k] * d[k, leave];
    d[i, leave] := sum
  end;

  { maximum element }
  imax := leave;
  aux := abs(d[imax, leave]);

  for i := leave + 1 to m do
  begin
    sum := d[i, leave];
    for k := 1 to leave - 1 do
      sum := sum - d[i, k] * d[k, leave];
    d[i, leave] := sum;
    if abs(sum) > aux then
    begin
      imax := i;
      aux := abs(sum)
    end
  end;

  if aux < tol1 then
    errormsg(singular);

  if imax <> leave then
  begin
    for k := 1 to m do
    begin
      aux := d[imax, k];
      d[imax, k] := d[leave, k];
      d[leave, k] := aux
    end;
    i := equation[imax];
    equation[imax] := equation[leave];
    equation[leave] := i;
    change[leave] := imax
  end;

  aux := d[leave, leave];
  for i := leave + 1 to m do
    d[i, leave] := d[i, leave] / aux;

  { column (leave + 1) .. m }
  for j := leave + 1 to m do
  begin
    for i := leave to j do
    begin
      sum := d[i, j];
      for k := 1 to i - 1 do
        sum := sum - d[i, k] * d[k, j];
      d[i, j] := sum
    end;

    { maximum element }
    imax := j;
    aux := abs(d[imax, j]);

    for i := j + 1 to m do
    begin
      sum := d[i, j];
      for k := 1 to j - 1 do
        sum := sum - d[i, k] * d[k, j];
      d[i, j] := sum;
      if abs(sum) > aux then
      begin
        imax := i;
        aux := abs(sum)
      end
    end;

    if aux < tol1 then
      errormsg(singular);

    if imax <> j then
    begin
      for k := 1 to m do
      begin
        aux := d[imax, k];
        d[imax, k] := d[j, k];
        d[j, k] := aux
      end;
      i := equation[imax];
      equation[imax] := equation[j];
      equation[j] := i;
      change[j] := imax
    end;

    aux := d[j, j];
    for i := j + 1 to m do
      d[i, j] := d[i, j] / aux
  end
end;

procedure solve(var b, x: rvector);
var
  i, k: integer;
  sum: single;

begin
  for i := 1 to m do
    x[i] := b[equation[i]];

  for i := 1 to m do
  begin
    sum := x[i];
    for k := 1 to i - 1 do
      sum := sum - d[i, k] * x[k];
    x[i] := sum
  end;

  for i := m downto 1 do
  begin
    sum := x[i];
    for k := i + 1 to m do
      sum := sum - d[i, k] * x[k];
    x[i] := sum / d[i, i]
  end
end;

procedure solvetransp(var b, x: rvector);
var
  i, k: integer;
  sum: single;

begin
  for i := 1 to m do
  begin
    sum := b[i];
    for k := 1 to i - 1 do
      sum := sum - d[k, i] * b[k];
    b[i] := sum / d[i, i]
  end;

  for i := m downto 1 do
  begin
    sum := b[i];
    for k := i + 1 to m do
      sum := sum - d[k, i] * b[k];
    b[i] := sum
  end;

  for i := 1 to m do
    x[equation[i]] := b[i]
end;

function gcdxy(x, y: integer): integer;
  { greatest common divisor for x, y }

begin
  while (x > 0) and (y > 0) do
  begin
    x := x mod y;
    if x > 0 then
      y := y mod x
  end;

  if x = 0 then
    gcdxy := y
  else
    gcdxy := x
end;

function knapsack(m, b: integer; a, o: ivector; var c, x: rvector): single;
{
  maximize f = c'x
  a'x <= b
  x >= 0
  0 < a[1] < a[2] < ... < a[m] , 0 < c[1] < c[2] < ... < c[m]
}
var
  a1, an, i, k, n, gcd, sum, last, ir, ar, xr: integer;
  cr: single;
  index: array [0..maxks] of integer;
  f: array [0..maxks] of single;
  v: rvector;

begin
  { sort vector c according to vector a }
  for i := 1 to m do
    v[i] := c[o[i]];

  { ensure increasing value for vector c components }
  i := 1;
  while i < m do
  begin
    for k := i + 1 to m do
      if v[i] >= v[k] then
        v[k] := 0.0;
    i := i + 1;
    while (v[i] = 0.0) and (i < m) do
      i := i + 1
  end;

  { shrinks vectors o, c, a }
  n := 0;
  for i := 1 to m do
    if v[i] > 0.0 then
    begin
      n := n + 1;
      o[n] := o[i];
      c[n] := v[i];
      a[n] := a[i]
    end;

  { calculates the greatest common divisor of vector a components }
  i := 1;
  gcd := a[1];
  while (i < n) and (gcd > 1) do
  begin
    i := i + 1;
    gcd := gcdxy(gcd, a[i])
  end;

  { division by the greatest common divisor }
  if gcd > 1 then
  begin
    b := b div gcd;
    for i := 1 to n do
      a[i] := a[i] div gcd
  end;

  { constants }
  a1 := a[1];
  an := a[n];

  { initialize }
  cr := c[1] / a[1];
  ir := 1;
  for i := 2 to n do
    if c[i] / a[i] > cr then
    begin
      cr := c[i] / a[i];
      ir := i
    end;
  ar := a[ir];
  cr := c[ir];

  for k := 0 to b + an do
  begin
    index[k] := 0;
    f[k] := 0.0
  end;

  { f(k) for 0 <= k <= (a1 - 1) }
  for k := 0 to a1 - 1 do
    for i := 1 to n do
      if c[i] >= f[k + a[i]] then
      begin
        f[k + a[i]] := c[i];
        index[k + a[i]] := i
      end;

  { f(k) for a1 <= k <= (ar - 1) }
  for k := a1 to ar - 1 do
    for i := index[k] to n do
      if c[i] + f[k] >= f[k + a[i]] then
      begin
        f[k + a[i]] := c[i] + f[k];
        index[k + a[i]] := i
      end;

  { f(k) for ar <= k <= b }
  sum := 0;
  k := ar - 1;
  while (k < b) and (sum < an) do
  begin
    k := k + 1;

    for i := index[k] to n do
      if c[i] + f[k] >= f[k + a[i]] then
      begin
        f[k + a[i]] := c[i] + f[k];
        index[k + a[i]] := i
      end;

    if cr + f[k - ar] = f[k] then
      sum := sum + 1
    else
      sum := 0
  end;
  last := k;

  { solution }
  xr := 0;
  for i := 1 to m do
    x[i] := 0;

  if last < b then
  begin
    xr := (b - last + an) div ar;
    b := b - xr * ar;
    x[o[ir]] := xr
  end;

  k := b;
  while k > (a1 - 1) do
  begin
    x[o[index[k]]] := x[o[index[k]]] + 1;
    k := k - a[index[k]]
  end;

  knapsack := f[b] + xr * cr
end;

function notoptimal(var a: rvector; var enter: integer): boolean;
{
  calculate maximum [-df/dxj for lower, df/dxj for upper]
  df/dxj = cj - p' * aj, where aj column of a
  df/dxj = width - (p + r)' * aj
  df/dxj = -p(j), 1 <= j <= nd
  df/dxj = -p(j - m2) * a(1,j) - p(j) * a(2,j), nd < j <= m
}
var
  i, j: integer;
  aux, tmax: single;

begin
  { search in non basic }
  tmax := minreal;
  enter := 0;

  for j := 1 to nd do
    if status[j] <> bas then
    begin
      if status[j] = upp then
        aux := -p[j]
      else
        aux := p[j];
      if aux > tmax then
      begin
        tmax := aux;
        enter := j
      end
    end;

  for j := nd + 1 to m do
    if status[j] <> bas then
    begin
      if status[j] = upp then
        aux := -p[j - m2] * aa[1, j] - p[j] * aa[2, j]
      else
        aux := p[j - m2] * aa[1, j] + p[j] * aa[2, j];
      if aux > tmax then
      begin
        tmax := aux;
        enter := j
      end
    end;

  if tmax > tol2 then
  begin
    for i := 1 to m do
      a[i] := 0.0;
    if enter <= nd then
      a[enter] := 1.0
    else
    begin
      a[enter - m2] := aa[1, enter];
      a[enter] := aa[2, enter]
    end
  end
  else
  begin
    for i := 1 to m do
      v[i] := p[i] + r[i];
    tmax := knapsack(m, widthks, wks, o, v, a) - width;
    enter := 0
  end;

  notoptimal := tmax > tol2
end;

procedure simplex;
var
  i, j, leave, enter: integer;
  aux, dx: single;
  leftstatus: statustype;

begin
  while nocutoff and notoptimal(a, enter) do
  begin
    iteration := iteration + 1;

    { evaluate q }
    v := a;
    solve(v, q);
    if (enter > 0) and (status[enter] = upp) then
      for i := 1 to m do
        q[i] := -q[i];

    { find dx, leave }
    dx := upper[enter];
    leave := 0;
    if status[enter] = low then
      status[enter] := upp
    else
      status[enter] := low;

    for i := 1 to m do
      if abs(q[i]) > tol3 then
      begin
        if q[i] > 0.0 then
          aux := xb[i] / q[i]
        else
        begin
          aux := upper[basecol[i]];
          if aux <> maxreal then
            aux := (xb[i] - aux) / q[i]
        end;

        if aux < dx then
        begin
          dx := aux;
          leave := i
        end
      end;

    { tests unlimited objective }
    if dx = maxreal then
      errormsg(unbounded);

    if leave > 0 then
    begin
      if q[leave] < 0.0 then
        leftstatus := upp
      else
        leftstatus := low;

      { update status, basecol, base, d, cb }
      update(leftstatus, enter, leave, a);

      { evaluate p }
      v := cb;
      solvetransp(v, p)
    end;

    { update vector xb - search in non basic at upp }
    v := b;

    for j := 1 to nd do
      if status[j] = upp then
        v[j] := v[j] - upper[j];

    for j := nd + 1 to m do
      if status[j] = upp then
      begin
        v[j - m2] := v[j - m2] - aa[1, j];
        v[j] := v[j] - aa[2, j]
      end;

    solve(v, xb);

    { cutoff }
    step := step + 1;
    if step > maxstep then
    begin
      step := 1;
      aux := 0.0;
      for i := 1 to m do
        aux := aux + cb[i] * xb[i];
      nocutoff := (aux - obj) / obj > tol4;
      obj := aux
    end
  end
end;

procedure input;
var
  k: integer;
  io: text;

begin
  assign(io, 'input.txt');
  reset(io);

  { roll width }
  read(io, width);
  readln(io);

  read(io, prodprice);
  read(io, prodcost);
  read(io, scrapprice);
  readln(io);

  { number of orders }
  read(io, nd);
  readln(io);

  for k := 1 to nd do
    with demand[k] do
    begin
      read(io, width1);
      read(io, width2);
      read(io, quantity);
      read(io, alpha);
      read(io, theta);
      readln(io)
    end;

  close(io)
end;

procedure writeline(var io: text; n: integer);
begin
  while n > 0 do
  begin
    write(io, '-');
    n := n - 1
  end;
  writeln(io)
end;

procedure output;
const
  sp = ' ';

var
  i, j, k, pattern: integer;
  produced, supplied, waste, profit, efficiency, dfdq, variation: single;
  supply: rvector;
  io: text;

begin
  assign(io, 'output.txt');
  rewrite(io);

  { initialize }
  produced := 0.0;
  pattern := 0;
  profit := 0.0;
  for i := 1 to m do
    supply[i] := 0.0;

  for j := 1 to m do
    if (basecol[j] = 0) and (xb[j] * width > minweight) then
    begin
      pattern := pattern + 1;
      profit := profit + cb[j] * xb[j];

      { waste, supply / w }
      waste := 0.0;
      for i := 1 to m do
        if base[i, j] > 0.0 then
        begin
          waste := waste + base[i, j] * w[i];
          supply[i] := supply[i] + xb[j] * base[i, j]
        end;
      waste := width - waste;

      produced := produced + xb[j] * width;

      writeln(io);
      writeline(io, 22);
      write(io, '     weight    waste %');
      writeln(io);

      write(io, (xb[j] * width): 11: 1);
      write(io, sp: 5);
      write(io, (waste / width * 100): 6: 2);
      writeln(io);

      writeln(io);
      write(io, ' cuts     width');
      writeln(io);
      for i := 1 to m do
        if base[i, j] > 0.0 then
        begin
          write(io, trunc(0.5 + base[i, j]): 5);
          write(io, sp: 5);
          write(io, w[i]: 5);
          writeln(io)
        end
    end;

  { supply, supplied }
  for i := 1 to m do
    supply[i] := supply[i] * w[i];
  supplied := 0.0;
  for i := 1 to m do
    supplied := supplied + supply[i];

  profit := (scrapprice - prodcost) * profit / 1000;
  efficiency := supplied / produced * 100;

  writeln(io);
  writeline(io, 39);

  write(io, '| roll width           |     ');
  write(io, width: 5);
  write(io, '    |');
  writeln(io);
  writeline(io, 39);

  write(io, '| number of orders     |     ');
  write(io, nd: 5);
  write(io, '    |');
  writeln(io);
  writeline(io, 39);

  write(io, '| number of patterns   |     ');
  write(io, pattern: 5);
  write(io, '    |');
  writeln(io);
  writeline(io, 39);

  write(io, '| produced weight      | ');
  write(io, produced: 11: 1);
  write(io, '  |');
  writeln(io);
  writeline(io, 39);

  write(io, '| harnessed weight     | ');
  write(io, supplied: 11: 1);
  write(io, '  |');
  writeln(io);
  writeline(io, 39);

  write(io, '| efficiency %         |       ');
  write(io, efficiency: 6: 2);
  write(io, ' |');
  writeln(io);
  writeline(io, 39);

  writeln(io);
  writeline(io, 39);

  write(io, '| paper price          |  ');
  write(io, prodprice: 11: 2);
  write(io, ' |');
  writeln(io);
  writeline(io, 39);

  write(io, '| paper cost           |  ');
  write(io, prodcost: 11: 2);
  write(io, ' |');
  writeln(io);
  writeline(io, 39);

  write(io, '| scrap price          |  ');
  write(io, scrapprice: 11: 2);
  write(io, ' |');
  writeln(io);
  writeline(io, 39);

  write(io, '| profit (thousands $) |  ');
  write(io, profit: 11: 2);
  write(io, ' |');
  writeln(io);
  writeline(io, 39);

  writeln(io);
  writeline(io, 71);
  write(io, '|       width |      weight | variation % |    $/weight | price ratio |');
  writeln(io);
  writeline(io, 71);

  for k := 1 to m1 do
    with demand[k] do
    begin
      dfdq := (scrapprice - prodcost) * p[k] * (1.0 + alpha) / w[k];
      variation := 100 * (supply[k] / quantity - 1.0);

      write(io, '|       ');
      write(io, w[k]: 5);
      write(io, ' | ');
      write(io, supply[k]: 11: 1);
      write(io, ' |      ');
      write(io, variation: 6: 2);
      write(io, ' | ');
      write(io, dfdq: 11: 2);
      write(io, ' |        ');
      write(io, theta: 4: 2);
      write(io, ' |');
      writeln(io);

      writeline(io, 71)
    end;

  for k := m1 + 1 to nd do
    with demand[k] do
    begin
      dfdq := (scrapprice - prodcost) * p[k] * (1.0 + alpha) / w[k];
      variation := 100 * ((supply[k] + supply[k + m2]) / quantity - 1.0);

      write(io, '|       ');
      write(io, w[k]: 5);
      write(io, ' | ');
      write(io, supply[k]: 11: 1);
      write(io, ' |      ');
      write(io, variation: 6: 2);
      write(io, ' | ');
      write(io, dfdq: 11: 2);
      write(io, ' |        ');
      write(io, theta: 4: 2);
      write(io, ' |');
      writeln(io);

      write(io, '|       ');
      write(io, w[k + m2]: 5);
      write(io, ' | ');
      write(io, supply[k + m2]: 11: 1);
      write(io, ' |             |             |             |');
      writeln(io);

      writeline(io, 71)
    end;

  close(io)
end;

procedure setup;
var
  i, j, k, imin, min, gcd: integer;
  aux1, aux2: single;

begin
  { number of orders with specified fiber }
  m1 := 0;
  for k := 1 to nd do
    if demand[k].width2 = 0 then
      m1 := m1 + 1;

  { number of orders with free fiber }
  m2 := nd - m1;

  m := m1 + m2 + m2;
  if m > maxm then
    errormsg(outbound1);

  { initialize }
  for j := 1 to m do
  begin
    aa[1, j] := 0.0;
    aa[2, j] := 0.0
  end;

  { initialize w, r, b, aa, upper }
  upper[0] := maxreal;
  aux1 := prodprice / (prodcost - scrapprice);
  aux2 := scrapprice / (prodcost - scrapprice);

  for k := 1 to m1 do
    with demand[k] do
    begin
      w[k] := width1;
      r[k] := (theta * aux1 - aux2) * width1;
      b[k] := quantity * (1.0 + alpha) / width1;
      upper[k] := 2.0 * alpha * quantity / width1
    end;

  for k := m1 + 1 to nd do
    with demand[k] do
    begin
      w[k] := width1;
      r[k] := (theta * aux1 - aux2) * width1;
      b[k] := quantity * (1.0 + alpha) / width1;
      upper[k] := 2.0 * alpha * quantity / width1;

      w[k + m2] := width2;
      r[k + m2] := (theta * aux1 - aux2) * width2;
      b[k + m2] := 0.0;
      upper[k + m2] := 1.0;

      aa[1, k + m2] := quantity * (1.0 + alpha) / width1;
      aa[2, k + m2] := -quantity * (1.0 + alpha) / width2
    end;

  { initialize status }
  for j := 0 to nd do
    status[j] := low;
  for j := nd + 1 to m do
    status[j] := bas;

  { initialize base, basecol, cb }
  for j := 1 to nd do
  begin
    for i := 1 to m do
      base[i, j] := 0.0;
    base[j, j] := width div w[j];
    basecol[j] := 0;
    cb[j] := width - r[j] * base[j, j]
  end;

  for j := nd + 1 to m do
  begin
    for i := 1 to m do
      base[i, j] := 0.0;
    base[j - m2, j] := aa[1, j];
    base[j, j] := aa[2, j];
    basecol[j] := j;
    cb[j] := 0.0
  end;

  { initialize d, equation, change, p, xb }
  for i := 1 to m do
    for j := 1 to m do
      d[i, j] := base[i, j];
  for i := 1 to m do
  begin
    equation[i] := i;
    change[i] := 0;
    p[i] := cb[i] / d[i, i];
    xb[i] := b[i] / d[i, i]
  end;

  { initialize widthks }
  widthks := width;

  { sort wks }
  for i := 1 to m do
  begin
    wks[i] := w[i];
    o[i] := i
  end;
  for i := 1 to m - 1 do
  begin
    imin := i;
    min := wks[i];

    for k := i + 1 to m do
      if wks[k] < min then
      begin
        imin := k;
        min := wks[k]
      end;

    if i <> imin then
    begin
      wks[imin] := wks[i];
      wks[i] := min;
      min := o[imin];
      o[imin] := o[i];
      o[i] := min
    end
  end;

  { calculates the greatest common divisor of vector wks components }
  i := 1;
  gcd := wks[1];
  while (i < m) and (gcd > 1) do
  begin
    i := i + 1;
    gcd := gcdxy(gcd, wks[i])
  end;

  { division by the greatest common divisor }
  if gcd > 1 then
  begin
    widthks := widthks div gcd;
    for i := 1 to m do
      wks[i] := wks[i] div gcd
  end;

  if (widthks + wks[m]) > maxks then
    errormsg(outbound2);

  { initialize step, nocutoff, obj, iteration }
  step := 1;
  nocutoff := true;
  obj := 0.0;
  for i := 1 to m do
    obj := obj + cb[i] * xb[i];
  iteration := 0
end;

begin
  input;

  setup;

  simplex;

  output
end.
